﻿using HarmonyLib;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using StardewModdingAPI;
using StardewValley;
using System;
using System.Collections.Generic;
using System.Reflection.Emit;
using System.Reflection;
using System.Linq;

namespace FriendshipTweaks
{
    public partial class ModEntry
    {
        //private ModConfig config;
        
        [HarmonyPatch(typeof(Utility), nameof(Utility.GetMaximumHeartsForCharacter))]
        public class Utility_GetMaximumHeartsForCharacter_Patch
        {
            public static bool Prefix(Character character, ref int __result)
            {
                if (!Config.ModEnabled || character is not NPC)
                    return true;

                if (!Game1.player.friendshipData.TryGetValue(character.Name, out Friendship f) || !f.IsMarried())
                    return true;
                //f.Points = 250 * Config.MaxHearts;
                __result = Config.MaxHearts;
                return false;
            }
        }

        [HarmonyPatch(typeof(Farmer), nameof(Farmer.changeFriendship))]
        public class Farmer_changeFriendship_Patch
        {
            public static void Prefix(ref int amount)
            {
                if (!Config.ModEnabled)
                    return;
                amount = (int)Math.Round(amount * (Math.Sign(amount) > 0 ? Config.IncreaseModifier : Config.DecreaseModifier));
            }
        }
        //[HarmonyPatch(typeof(Friendship), nameof(Friendship.Points))]
        //[HarmonyPatch(MethodType.Setter)]
        public class Friendship_Points_Patch
        {
            public static void Prefix(Friendship __instance, ref int value)
            {
                if (!Config.ModEnabled)
                    return;
                var change = value - __instance.Points;
                change = (int)Math.Round(change * (Math.Sign(change) > 0 ? Config.IncreaseModifier : Config.DecreaseModifier));
                value = __instance.Points + change;
            }
        }

        public static float CustomBirthdayMult()
        {
            if (!Config.ModEnabled)
            {
                return 8f;
            }
            return Config.BirthdayMultiplier;
        }

        public static float CustomWinterStarMult()
        {
            if (!Config.ModEnabled)
            {
                return 5f;
            }
            return Config.WinterStarMultiplier;
        }

        public static int CustomStardropTeaMult()
        {
            if (!Config.ModEnabled)
            {
                return 750;
            }
            return (int)System.Math.Round(((750/3) * Config.StardropTeaMultiplier), MidpointRounding.AwayFromZero);
        }

        public class New_Patches
        {
            private static IMonitor Monitor { get; set; } = null;

            static MethodInfo BirthdayMultMethod = SymbolExtensions.GetMethodInfo(() => ModEntry.CustomBirthdayMult());
            static MethodInfo WinterStarMultMethod = SymbolExtensions.GetMethodInfo(() => ModEntry.CustomWinterStarMult());
            static MethodInfo StardropteaMultMethod = SymbolExtensions.GetMethodInfo(() => ModEntry.CustomStardropTeaMult());
            
            public static IEnumerable<CodeInstruction> GiftPatch(IEnumerable<CodeInstruction> instructions)
            {
                foreach (var instruction in instructions)
                {
                    if (instruction.LoadsConstant(8f))
                    {
                        //Monitor.Log($"Attempted to change Birthday friendshipChangeMultiplier from 8 to {(int)CustomBirthdayMult()}. This is listed as an Error for debugging purposes.", LogLevel.Error);
                        yield return new CodeInstruction(OpCodes.Call, BirthdayMultMethod);
                        continue;
                    }
                    if (instruction.LoadsConstant(750))
                    {
                        //Monitor.Log($"Attempted to change Stardrop Tea maximum changeFriendship cap from 750 to {CustomStardropTeaMult()}. This is listed as an Error for debugging purposes.", LogLevel.Error);
                        yield return new CodeInstruction(OpCodes.Call, StardropteaMultMethod);
                        continue;
                    }

                    yield return instruction;
                }
            }

            public static IEnumerable<CodeInstruction> WinterStarPatch(IEnumerable<CodeInstruction> instructions)
            {
                foreach (var instruction in instructions)
                {
                    if (instruction.LoadsConstant(5f))
                    {
                        //Monitor.Log($"Attempted to change Winter Star friendshipChangeMultiplier from 5 to {(int)CustomWinterStarMult()}. This is listed as an Error for debugging purposes.", LogLevel.Error);
                        yield return new CodeInstruction(OpCodes.Call, WinterStarMultMethod);
                        continue;
                    }

                    yield return instruction;
                }
            }
        }
    }
}